﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Dynamic2D;

using Game = Dynamic2D.Game;

namespace Stardrive
{
    public class EnemyBase : ShipBase
    {
        public float speed;
        public float turnRate;

        public Vector2[] shotLocations;
        public BulletBase shotBase;
        public SoundEffect shotEffect;
        public float shotRange;
        public float shotSpeed;
        public float shotRefireDelay;

        public float shotAngle = 0.5f;

        public WreckageBase wreckageBase;
        public Sprite upgradeSprite;
        public Upgrade upgrade;
        public float upgradeDropChance = 0.05f;
    }

    public class Enemy : Ship
    {
        public float shotRefireTimer = 0;

        bool bShoot = false;

        public Enemy(EnemyBase BaseEnemy)
            : base(BaseEnemy)
        {
        }

        public override void Update(GameTime gameTime)
        {
            EnemyBase BaseEnemy = (EnemyBase)BaseObject;

            bShoot = false;

            if (health > 0 && ((StardriveGame)game).Player.IsActive())
            {
                Vector2 ToPlayer = ((StardriveGame)game).Player.position - position;
                float desiredRotation = Game.AngleFromVector2(ToPlayer);
                desiredRotation = MathHelper.WrapAngle(desiredRotation - rotation);
                rotVelocity = MathHelper.Clamp(desiredRotation, -BaseEnemy.turnRate, BaseEnemy.turnRate);
                velocity = Game.Vector2FromAngle(rotation) * BaseEnemy.speed;

                if (ToPlayer.LengthSquared() < BaseEnemy.shotRange * BaseEnemy.shotRange
                    && Math.Abs(desiredRotation) < BaseEnemy.shotAngle)
                {
                    bShoot = true;
                }

                shotRefireTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;

                if (bShoot)
                {
                    Shoot();
                }
            }

 	        base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);
        }

        public override void Destroy()
        {
            base.Destroy();

            EnemyBase BaseEnemy = (EnemyBase)BaseObject;
            if ((float)game.random.NextDouble() < BaseEnemy.upgradeDropChance)
            {
                if (BaseEnemy.upgradeSprite.texture != null
                    || BaseEnemy.upgrade.sprite.texture != null)
                {
                    Wreckage Wreckage = BaseEnemy.wreckageBase.ConstructInstance<Wreckage>();
                    Wreckage.upgrade = BaseEnemy.upgrade;
                    Wreckage.faction = faction;
                    Wreckage.position = position;
                    Wreckage.rotation = rotation;
                    Wreckage.velocity = velocity / 4;
                    //Vector2 delta = (Wreckage.position - position);
                    //delta.Normalize();
                    //Wreckage.velocity += delta * 200;
                    Wreckage.rotVelocity = MathHelper.Lerp(-5, 5, (float)game.random.NextDouble());
                    Wreckage.AddToGame(game, Game.LayerId.World);
                    Wreckage.sprite = BaseEnemy.upgrade.sprite; //BaseEnemy.upgradeSprite;
                    Wreckage.explosion.Animation = explosion.Animation;
                }
            }
        }

        private void Shoot()
        {
            if (shotRefireTimer <= 0)
            {
                EnemyBase BaseEnemy = (EnemyBase)BaseObject;
                shotRefireTimer = BaseEnemy.shotRefireDelay;

                BaseEnemy.shotEffect.Play();

                foreach (Vector2 shotLocation in BaseEnemy.shotLocations)
                {
                    Bullet shot = BaseEnemy.shotBase.ConstructInstance<Bullet>();
                    shot.owner = this;
                    shot.faction = faction;
                    shot.position = TransformLocalToWorld(shotLocation);
                    shot.rotation = rotation;
                    shot.velocity = velocity + Game.Vector2FromAngle(rotation) * BaseEnemy.shotSpeed;
                    shot.range = BaseEnemy.shotRange;
                    shot.AddToGame(game, Game.LayerId.World);
                }
            }
        }
    }
}
