﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Dynamic2D;

using Game = Dynamic2D.Game;

namespace Stardrive
{
    public struct Upgrade
    {
        public Sprite sprite;
        public BulletBase shotBase;
        public SoundEffect shotEffect;
        public float shotRange;
        public float shotSpeed;
        public float shotRefireDelay;

        public Sprite thrust;
        public Vector2 thrustPosition;
    }

    public struct UpgradeEntry
    {
        public Upgrade upgrade;
        public Vector2 relativeLocation;

        public float shotRefireTimer;
    }

    public class PlayerShipBase : ShipBase
    {
        public float speed;
        public float turnRate;

        public WreckageBase wreckageBase;

        public Vector2 shotLocation;
        public GameObjectBase shotBase;
        public SoundEffect shotEffect;
        public float shotRange;
        public float shotSpeed;
        public float shotRefireDelay;
    }

    public class PlayerShip : Ship
    {
        public List<UpgradeEntry> upgrades = new List<UpgradeEntry>();

        public float shotRefireTimer = 0;

        public PlayerShip(PlayerShipBase BasePlayerShip)
            : base(BasePlayerShip)
        {
        }

        public override void Update(GameTime gameTime)
        {
            PlayerShipBase BasePlayerShip = (PlayerShipBase)BaseObject;

            bThrust = false;
            bool bShoot = false;

            if (health > 0)
            {
                velocity = Vector2.Zero;
                rotVelocity = 0;

                Vector2 Left = game.gamePadState.ThumbSticks.Left;
                //Left.Y = -Left.Y;
                //Vector2 Right = game.gamePadState.ThumbSticks.Right;
                //Right.Y = -Right.Y;

                if (Left != Vector2.Zero
                    || game.gamePadState.Triggers.Right != 0
                    || game.gamePadState.IsButtonDown(Buttons.A))
                {
                    if (Left != Vector2.Zero)
                    {
                        rotVelocity = Left.X * BasePlayerShip.turnRate;
                    }
                    if (game.gamePadState.Triggers.Right > 0)
                    {
                        velocity = game.gamePadState.Triggers.Right * Dynamic2D.Game.Vector2FromAngle(rotation) * BasePlayerShip.speed;
                        bThrust = true;
                    }

                    if (game.gamePadState.IsButtonDown(Buttons.A))
                    {
                        bShoot = true;
                    }
                }
                else
                {
                    if (game.keyboardState.IsKeyDown(Keys.Left))
                    {
                        rotVelocity = -BasePlayerShip.turnRate;
                    }
                    if (game.keyboardState.IsKeyDown(Keys.Right))
                    {
                        rotVelocity = BasePlayerShip.turnRate;
                    }
                    if (game.keyboardState.IsKeyDown(Keys.Up))
                    {
                        velocity = Dynamic2D.Game.Vector2FromAngle(rotation) * BasePlayerShip.speed;
                        bThrust = true;
                    }
                    if (game.keyboardState.IsKeyDown(Keys.Space))
                    {
                        bShoot = true;
                    }
                }

                if (shotRefireTimer > 0)
                {
                    shotRefireTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                }

                for (int i = 0; i < upgrades.Count; i++)
                {
                    if (upgrades[i].shotRefireTimer > 0)
                    {
                        UpgradeEntry upgrade = upgrades[i];
                        upgrade.shotRefireTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                        upgrades[i] = upgrade;
                    }
                }

                if (bShoot)
                {
                    Shoot();
                }
            }
            else
            {
                if (upgrades.Count > 0 && explosion.Frame >= 3)
                {
                    foreach (UpgradeEntry upgrade in upgrades)
                    {
                        Wreckage Wreckage = BasePlayerShip.wreckageBase.ConstructInstance<Wreckage>();
                        Wreckage.upgrade = upgrade.upgrade;
                        Wreckage.faction = faction;
                        Wreckage.position = TransformLocalToWorld(upgrade.relativeLocation);
                        Wreckage.rotation = rotation;
                        Wreckage.velocity = velocity;
                        Vector2 delta = (Wreckage.position - position);
                        delta.Normalize();
                        Wreckage.velocity += delta * Math.Abs(upgrade.relativeLocation.X) * 12.5f;
                        Wreckage.rotVelocity = MathHelper.Lerp(-5, 5, (float)game.random.NextDouble());
                        Wreckage.AddToGame(game, Game.LayerId.World);
                        Wreckage.sprite = upgrade.upgrade.sprite;
                        Wreckage.explosion.Animation = explosion.Animation;
                    }
                    upgrades.Clear();
                }
            }

 	        base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            if (health > 0)
            {
                foreach (UpgradeEntry upgrade in upgrades)
                {
                    game.DrawObject(upgrade.upgrade.sprite, TransformLocalToWorld(upgrade.relativeLocation), rotation, scale, layerDepth);

                    if (bThrust)
                    {
                        game.DrawObject(upgrade.upgrade.thrust, TransformLocalToWorld(upgrade.relativeLocation + upgrade.upgrade.thrustPosition), rotation, scale, layerDepth);
                    }
                }
            }
            else
            {
                foreach (UpgradeEntry upgrade in upgrades)
                {
                    game.DrawObject(upgrade.upgrade.sprite, TransformLocalToWorld(upgrade.relativeLocation), rotation, scale, layerDepth);
                }
            }
        }

        public override void NotifyCollision(GameObject other)
        {
            if (other is Wreckage)
            {
                Wreckage otherWreckage = (Wreckage)other;
                UpgradeEntry newUpgrade;
                newUpgrade.upgrade = otherWreckage.upgrade;
                newUpgrade.shotRefireTimer = 0;
                newUpgrade.relativeLocation = new Vector2((upgrades.Count / 2) * 10 + 16, 0);
                if (upgrades.Count % 2 == 1)
                {
                    newUpgrade.relativeLocation = -newUpgrade.relativeLocation;
                }
                upgrades.Add(newUpgrade);

                otherWreckage.RemoveFromGame();
            }
        }

        private void Shoot()
        {
            if (shotRefireTimer <= 0)
            {
                PlayerShipBase BasePlayerShip = (PlayerShipBase)BaseObject;
                shotRefireTimer = BasePlayerShip.shotRefireDelay;

                BasePlayerShip.shotEffect.Play();

                Bullet shot = BasePlayerShip.shotBase.ConstructInstance<Bullet>();
                shot.owner = this;
                shot.faction = faction;
                shot.position = TransformLocalToWorld(BasePlayerShip.shotLocation);
                shot.rotation = rotation;
                shot.velocity = velocity + Game.Vector2FromAngle(rotation) * BasePlayerShip.shotSpeed;
                shot.range = BasePlayerShip.shotRange;
                shot.AddToGame(game, Game.LayerId.World);
            }

            for (int i = 0; i < upgrades.Count; i++)
            {
                if (upgrades[i].shotRefireTimer <= 0)
                {
                    UpgradeEntry upgrade = upgrades[i];
                    upgrade.shotRefireTimer = upgrade.upgrade.shotRefireDelay;
                    upgrades[i] = upgrade;

                    Bullet shot = upgrade.upgrade.shotBase.ConstructInstance<Bullet>();
                    shot.owner = this;
                    shot.position = TransformLocalToWorld(upgrade.relativeLocation);
                    shot.rotation = rotation;
                    shot.velocity = velocity + Dynamic2D.Game.Vector2FromAngle(rotation) * upgrade.upgrade.shotSpeed;
                    shot.range = upgrade.upgrade.shotRange;
                    shot.AddToGame(game, Game.LayerId.World);

                    if (upgrade.upgrade.shotEffect != null)
                    {
                        upgrade.upgrade.shotEffect.Play();
                    }
                }
            }
        }
    }
}
