﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Dynamic2D;

using Game = Dynamic2D.Game;

namespace Stardrive
{
    public class ShipBase : GameObjectBase
    {
        public StardriveGame.Faction faction;

        public float health;
        public Animation explosion;
        public SoundEffect explosionSound;

        public Sprite thrust;
        public Vector2 thrustPosition = new Vector2(0, 7);
    }

    public class Ship : GameObject
    {
        public StardriveGame.Faction faction; 
        
        public float health;
        public AnimationInstance explosion;

        public bool bThrust = true;

        public Ship(ShipBase BaseShip)
            : base(BaseShip)
        {
            faction = BaseShip.faction;
        }

        public override void PostAddToGame()
        {
            base.PostAddToGame();

            ShipBase BaseShip = (ShipBase)BaseObject;
            health = BaseShip.health;
            explosion = new AnimationInstance(BaseShip.explosion);
        }

        public override void Update(GameTime gameTime)
        {
 	        base.Update(gameTime);

            if (health <= 0)
            {
                if (explosion.Animation != null)
                {
                    explosion.Update(gameTime.ElapsedGameTime);
                }

                if (explosion.Animation == null || explosion.HasEnded())
                {
                    RemoveFromGame();
                }
            }
        }

        public override void Draw(GameTime gameTime)
        {
            if (health > 0)
            {
                base.Draw(gameTime);

                ShipBase BaseShip = (ShipBase)BaseObject;
                if (bThrust && BaseShip.thrust.texture != null)
                {
                    game.DrawObject(BaseShip.thrust, TransformLocalToWorld(BaseShip.thrustPosition), rotation, scale, layerDepth);
                }
            }
            else
            {
                if (explosion.Frame < 3)
                {
                    base.Draw(gameTime);
                }
                explosion.Draw(game, position, rotation, scale, layerDepth);
            }
        }

        public virtual void TakeDamage(float damage)
        {
            if (health > 0)
            {
                health -= damage;
                if (health <= 0)
                {
                    Destroy();
                }
            }
        }

        public virtual void Destroy()
        {
            ShipBase BaseShip = (ShipBase)BaseObject;
            if (BaseShip.explosionSound != null)
            {
                BaseShip.explosionSound.Play();
            }
        }
    }
}
