using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Dynamic2D;

namespace Stardrive
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class StardriveGame : Dynamic2D.Game
    {
        public enum GameState
        {
            Startup,
            Menu,
            Intro,
            Game,
            Death,
        }

        public GameState state = GameState.Startup;

        SimpleBackgroundBase BackgroundBase;

        LogoBoxBase StarDriveLogoBase;
        LogoBox StarDriveLogo;

        MenuBase MainMenuBase;
        Menu MainMenu;

        ChatBoxBase IntroChatBoxBase;
        ChatBox IntroChatBox;

        struct IntroText
        {
            public float time;
            public string speaker;
            public string text;
        }
        IntroText[] Intro;
        int IntroCursor;
        float IntroTimer;
        const float IntroLeadOut = 2.0f;

        ShipBase IntroShipBase;
        Ship IntroShip;

        public enum Faction
        {
            Player,
            Enemy,
        }

        public WreckageBase BaseWreckage;

        public PlayerShipBase PlayerBase;
        public PlayerShip Player;

        public EnemyBase BeamEnemyBase;
        public Upgrade BeamGunUpgrade;

        public EnemyBase StarEnemyBase;
        public Upgrade StarGunUpgrade;

        public EnemyBase WaveEnemyBase;
        public Upgrade WaveGunUpgrade;

        public float EnemySpawnTimer = 0;
        public float EnemySpawnDelay = 2.0f;

        public EnemyBase[] Enemies;
        public float[] EnemyTimes;
        public int EnemyWave = 0;

        public StardriveGame()
        {
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            BackgroundBase = new SimpleBackgroundBase();
            BackgroundBase.GameObjectType = typeof(SimpleBackground);
            BackgroundBase.sprite = new Sprite(Content.Load<Texture2D>(@"Background"), Vector2.Zero);
            BackgroundBase.sprite.sourceRect.Width = GraphicsDevice.Viewport.Width;
            BackgroundBase.sprite.sourceRect.Height = GraphicsDevice.Viewport.Height;

            SimpleBackground Background = BackgroundBase.ConstructInstance<SimpleBackground>();
            Background.AddToGame(this, LayerId.Background);

            StarDriveLogoBase = new LogoBoxBase();
            StarDriveLogoBase.GameObjectType = typeof(LogoBox);
            StarDriveLogoBase.font = Content.Load<SpriteFont>(@"MenuFont");
            StarDriveLogoBase.border = new Box9(Content.Load<Texture2D>(@"ChatBox"), 4, 28, 4, 28);

            StarDriveLogo = StarDriveLogoBase.ConstructInstance<LogoBox>();
            StarDriveLogo.position = new Vector2(GraphicsDevice.Viewport.Width / 2, 100);
            StarDriveLogo.text = "StarDrive";
            
            MainMenuBase = new MenuBase();
            MainMenuBase.GameObjectType = typeof(Menu);
            MainMenuBase.font = Content.Load<SpriteFont>(@"MenuFont");
            MainMenuBase.menuItems = new string[2];
            MainMenuBase.menuItems[0] = @"New Game";
            MainMenuBase.menuItems[1] = @"Exit Game";
            //MainMenuBase.MenuSelector = new Sprite(Content.Load<Texture2D>(@"MenuSelector"));
            MainMenuBase.MenuSelectorString = @">";
            MainMenuBase.MenuSelectorOffset = -30;

            MainMenu = MainMenuBase.ConstructInstance<Menu>();
            MainMenu.position = new Vector2((GraphicsDevice.Viewport.Width - 300) / 2, 350);

            IntroChatBoxBase = new ChatBoxBase();
            IntroChatBoxBase.GameObjectType = typeof(ChatBox);
            IntroChatBoxBase.border = new Box9(Content.Load<Texture2D>(@"ChatBox"), 4, 28, 4, 28);
            IntroChatBoxBase.font = Content.Load<SpriteFont>(@"ChatFont");
            IntroChatBoxBase.charDelay = 0.07f;

            IntroChatBox = IntroChatBoxBase.ConstructInstance<ChatBox>();
            IntroChatBox.position = new Vector2((GraphicsDevice.Viewport.Width - 600) / 2, GraphicsDevice.Viewport.Height - 250);
            IntroChatBox.size = new Vector2(600, 180);

            Intro = new IntroText[14];
            Intro[0].time = 0.5f;
            Intro[0].speaker = "Mission Control:";
            Intro[0].text = "Testing Testing 1... 2... 3...\nTesting...";
            Intro[1].time = 1.0f;
            Intro[1].speaker = "Mission Control:";
            Intro[1].text = "Jumper 1, come in.";
            Intro[2].time = 0.2f;
            Intro[2].speaker = "Jumper 1:";
            Intro[2].text = "Get on with it control.";
            Intro[3].time = 1.0f;
            Intro[3].speaker = "Mission Control:";
            Intro[3].text = "Right.";
            Intro[4].time = 1.0f;
            Intro[4].speaker = "Mission Control:";
            Intro[4].text = "In 30 seconds, you will make history.\nYou will become the first person to visit\nanother star system.";
            Intro[5].time = 1.0f;
            Intro[5].speaker = "Mission Control:";
            Intro[5].text = "Once the jump drive engages, you will be\nALONE. The communications delay will be\nlonger than your lifetime.";
            Intro[6].time = 1.0f;
            Intro[6].speaker = "Mission Control:";
            Intro[6].text = "There will be no help coming.";
            Intro[7].time = 1.0f;
            Intro[7].speaker = "Mission Control:";
            Intro[7].text = "Try to come back in one piece.\nIf you can bring back anything interesting,\nwe'd appreciate it.";
            Intro[8].time = 1.0f;
            Intro[8].speaker = "Mission Control:";
            Intro[8].text = "Are you ready?";
            Intro[9].time = 1.0f;
            Intro[9].speaker = "Jumper 1:";
            Intro[9].text = "Always am.\nEngaging jump drive in 5 seconds.";
            Intro[10].time = 1.0f;
            Intro[10].speaker = "Jumper 1:";
            Intro[10].text = "4.";
            Intro[11].time = 1.0f;
            Intro[11].speaker = "Jumper 1:";
            Intro[11].text = "3.";
            Intro[12].time = 1.0f;
            Intro[12].speaker = "Jumper 1:";
            Intro[12].text = "2.";
            Intro[13].time = 1.0f;
            Intro[13].speaker = "Jumper 1:";
            Intro[13].text = "1.";

            IntroShipBase = new ShipBase();
            IntroShipBase.GameObjectType = typeof(Ship);
            IntroShipBase.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerShip"));
            IntroShipBase.collisionRadius = 8;
            IntroShipBase.faction = Faction.Player;
            IntroShipBase.health = 20;
            IntroShipBase.explosion = new Animation(Content.Load<Texture2D>(@"PlayerExplosion"), 1, 8, 8);
            IntroShipBase.explosionSound = Content.Load<SoundEffect>(@"Explosion");
            IntroShipBase.thrust = new Sprite(Content.Load<Texture2D>(@"PlayerThrust"), new Vector2(4, 0));

            IntroShip = IntroShipBase.ConstructInstance<Ship>();
            IntroShip.position = new Vector2(0, 0);
            IntroShip.velocity = new Vector2(0, -100);
            IntroShip.bThrust = true;

            BaseWreckage = new WreckageBase();
            BaseWreckage.GameObjectType = typeof(Wreckage);
            BaseWreckage.collisionRadius = 8;
            BaseWreckage.health = 20;

            PlayerBase = new PlayerShipBase();
            PlayerBase.GameObjectType = typeof(PlayerShip);
            PlayerBase.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerShip"));
            PlayerBase.collisionRadius = 8;
            PlayerBase.faction = Faction.Player;
            PlayerBase.health = 20;
            PlayerBase.speed = 320;
            PlayerBase.turnRate = 5;
            PlayerBase.explosion = new Animation(Content.Load<Texture2D>(@"PlayerExplosion"), 1, 8, 8);
            PlayerBase.explosionSound = Content.Load<SoundEffect>(@"Explosion");
            PlayerBase.thrust = new Sprite(Content.Load<Texture2D>(@"PlayerThrust"), new Vector2(4, 0));
            PlayerBase.wreckageBase = BaseWreckage;

            PlayerBase.shotBase = new BulletBase();
            PlayerBase.shotBase.GameObjectType = typeof(Bullet);
            PlayerBase.shotBase.animation = new Animation(Content.Load<Texture2D>(@"PlayerStarShot"), 1, 5, 8, true);
            PlayerBase.shotBase.collisionRadius = 6;
            PlayerBase.shotEffect = Content.Load<SoundEffect>(@"Star_Shoot");
            PlayerBase.shotLocation = new Vector2(0, -7);
            PlayerBase.shotRange = 300;
            PlayerBase.shotSpeed = 300;
            PlayerBase.shotRefireDelay = 0.2f;

            BeamGunUpgrade.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerBeamGun"));
            BeamGunUpgrade.shotBase = new BulletBase();
            BeamGunUpgrade.shotBase.GameObjectType = typeof(Bullet);
            BeamGunUpgrade.shotBase.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerBeamShot"));
            BeamGunUpgrade.shotBase.collisionRadius = 6;
            BeamGunUpgrade.shotBase.damage = 2;
            BeamGunUpgrade.shotEffect = Content.Load<SoundEffect>(@"Beam_Shoot");
            BeamGunUpgrade.shotRange = 300;
            BeamGunUpgrade.shotSpeed = 300;
            BeamGunUpgrade.shotRefireDelay = 0.2f;
            BeamGunUpgrade.thrust = new Sprite(Content.Load<Texture2D>(@"PlayerGunThrust"), new Vector2(4, 0));
            BeamGunUpgrade.thrustPosition = new Vector2(0, 7);

            StarGunUpgrade.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerStarGun"));
            StarGunUpgrade.shotBase = new BulletBase();
            StarGunUpgrade.shotBase.GameObjectType = typeof(Bullet);
            StarGunUpgrade.shotBase.animation = new Animation(Content.Load<Texture2D>(@"PlayerStarShot"), 1, 5, 8, true);
            StarGunUpgrade.shotBase.collisionRadius = 6;
            StarGunUpgrade.shotBase.damage = 4;
            StarGunUpgrade.shotEffect = Content.Load<SoundEffect>(@"Star_Shoot");
            StarGunUpgrade.shotRange = 400;
            StarGunUpgrade.shotSpeed = 100;
            StarGunUpgrade.shotRefireDelay = 0.3f;
            StarGunUpgrade.thrust = new Sprite(Content.Load<Texture2D>(@"PlayerGunThrust"), new Vector2(4, 0));
            StarGunUpgrade.thrustPosition = new Vector2(0, 7);

            WaveGunUpgrade.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerWaveGun"));
            WaveGunUpgrade.shotBase = new BulletBase();
            WaveGunUpgrade.shotBase.GameObjectType = typeof(Bullet);
            WaveGunUpgrade.shotBase.sprite = new Sprite(Content.Load<Texture2D>(@"PlayerWaveShot"));
            WaveGunUpgrade.shotBase.collisionRadius = 6;
            WaveGunUpgrade.shotBase.damage = 3;
            WaveGunUpgrade.shotEffect = Content.Load<SoundEffect>(@"Wave_Shoot");
            WaveGunUpgrade.shotRange = 600;
            WaveGunUpgrade.shotSpeed = 200;
            WaveGunUpgrade.shotRefireDelay = 0.1f;
            WaveGunUpgrade.thrust = new Sprite(Content.Load<Texture2D>(@"PlayerGunThrust"), new Vector2(4, 0));
            WaveGunUpgrade.thrustPosition = new Vector2(0, 7);

            BeamEnemyBase = new EnemyBase();
            BeamEnemyBase.GameObjectType = typeof(Enemy);
            BeamEnemyBase.sprite = new Sprite(Content.Load<Texture2D>(@"EnemyBeamGun"));
            BeamEnemyBase.collisionRadius = 16;
            BeamEnemyBase.faction = Faction.Enemy;
            BeamEnemyBase.health = 1;
            BeamEnemyBase.speed = 300;
            BeamEnemyBase.turnRate = 4;
            BeamEnemyBase.explosion = new Animation(Content.Load<Texture2D>(@"EnemyExplosion"), 1, 8, 8);
            BeamEnemyBase.explosionSound = Content.Load<SoundEffect>(@"Explosion_short");
            BeamEnemyBase.shotBase = new BulletBase();
            BeamEnemyBase.shotBase.GameObjectType = typeof(Bullet);
            BeamEnemyBase.shotBase.sprite = new Sprite(Content.Load<Texture2D>(@"EnemyBeamShot"));
            BeamEnemyBase.shotBase.collisionRadius = 6;
            BeamEnemyBase.shotEffect = Content.Load<SoundEffect>(@"Beam_Shoot");
            BeamEnemyBase.shotLocations = new Vector2[2];
            BeamEnemyBase.shotLocations[0] = new Vector2(-10, -7);
            BeamEnemyBase.shotLocations[1] = new Vector2(10, -7);
            BeamEnemyBase.shotRange = 300;
            BeamEnemyBase.shotSpeed = 300;
            BeamEnemyBase.shotRefireDelay = 0.4f;
            BeamEnemyBase.thrust = new Sprite(Content.Load<Texture2D>(@"EnemyThrust"), new Vector2(16, 0));
            BeamEnemyBase.thrustPosition = new Vector2(0, 4);
            BeamEnemyBase.upgrade = BeamGunUpgrade;
            BeamEnemyBase.upgradeDropChance = 0.12f;
            BeamEnemyBase.wreckageBase = BaseWreckage; 
            
            StarEnemyBase = new EnemyBase();
            StarEnemyBase.GameObjectType = typeof(Enemy);
            StarEnemyBase.sprite = new Sprite(Content.Load<Texture2D>(@"EnemyStarGun"));
            StarEnemyBase.collisionRadius = 16;
            StarEnemyBase.faction = Faction.Enemy;
            StarEnemyBase.health = 4;
            StarEnemyBase.speed = 320;
            StarEnemyBase.turnRate = 5;
            StarEnemyBase.explosion = new Animation(Content.Load<Texture2D>(@"EnemyExplosion"), 1, 8, 8);
            StarEnemyBase.explosionSound = Content.Load<SoundEffect>(@"Explosion_short");
            StarEnemyBase.shotBase = new BulletBase();
            StarEnemyBase.shotBase.GameObjectType = typeof(Bullet);
            StarEnemyBase.shotBase.animation = new Animation(Content.Load<Texture2D>(@"EnemyStarShot"), 1, 5, 8, true);
            StarEnemyBase.shotBase.collisionRadius = 6;
            StarEnemyBase.shotBase.damage = 2;
            StarEnemyBase.shotEffect = Content.Load<SoundEffect>(@"Star_Shoot");
            StarEnemyBase.shotLocations = new Vector2[2];
            StarEnemyBase.shotLocations[0] = new Vector2(-10, -7);
            StarEnemyBase.shotLocations[1] = new Vector2( 10, -7);
            StarEnemyBase.shotRange = 400;
            StarEnemyBase.shotSpeed = 100;
            StarEnemyBase.shotRefireDelay = 0.3f;
            StarEnemyBase.thrust = new Sprite(Content.Load<Texture2D>(@"EnemyThrust"), new Vector2(16, 0));
            StarEnemyBase.thrustPosition = new Vector2(0, 4);
            StarEnemyBase.upgrade = StarGunUpgrade;
            StarEnemyBase.upgradeDropChance = 0.1f;
            StarEnemyBase.wreckageBase = BaseWreckage;

            WaveEnemyBase = new EnemyBase();
            WaveEnemyBase.GameObjectType = typeof(Enemy);
            WaveEnemyBase.sprite = new Sprite(Content.Load<Texture2D>(@"EnemyWaveGun"));
            WaveEnemyBase.collisionRadius = 16;
            WaveEnemyBase.faction = Faction.Enemy;
            WaveEnemyBase.health = 20;
            WaveEnemyBase.speed = 350;
            WaveEnemyBase.turnRate = 6;
            WaveEnemyBase.explosion = new Animation(Content.Load<Texture2D>(@"EnemyExplosion"), 1, 8, 8);
            WaveEnemyBase.explosionSound = Content.Load<SoundEffect>(@"Explosion_short");
            WaveEnemyBase.shotBase = new BulletBase();
            WaveEnemyBase.shotBase.GameObjectType = typeof(Bullet);
            WaveEnemyBase.shotBase.sprite = new Sprite(Content.Load<Texture2D>(@"EnemyWaveShot"));
            WaveEnemyBase.shotBase.collisionRadius = 6;
            WaveEnemyBase.shotBase.damage = 4;
            WaveEnemyBase.shotEffect = Content.Load<SoundEffect>(@"Wave_Shoot");
            WaveEnemyBase.shotLocations = new Vector2[2];
            WaveEnemyBase.shotLocations[0] = new Vector2(-10, -7);
            WaveEnemyBase.shotLocations[1] = new Vector2(10, -7);
            WaveEnemyBase.shotRange = 300;
            WaveEnemyBase.shotSpeed = 200;
            WaveEnemyBase.shotRefireDelay = 0.1f;
            WaveEnemyBase.thrust = new Sprite(Content.Load<Texture2D>(@"EnemyThrust"), new Vector2(16, 0));
            WaveEnemyBase.thrustPosition = new Vector2(0, 4);
            WaveEnemyBase.upgrade = WaveGunUpgrade;
            WaveEnemyBase.upgradeDropChance = 0.05f;
            WaveEnemyBase.wreckageBase = BaseWreckage;

            Enemies = new EnemyBase[3];
            Enemies[0] = BeamEnemyBase;
            Enemies[1] = StarEnemyBase;
            Enemies[2] = WaveEnemyBase;

            EnemyTimes = new float[3];
            EnemyTimes[0] = 30;
            EnemyTimes[1] = 60;
            EnemyTimes[2] = float.MaxValue;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void PreUpdateGame(GameTime gameTime)
        {
            if (state == GameState.Game)
            {
                if ((gamePadState.Buttons.Start == ButtonState.Pressed && prevGamePadState.Buttons.Start != ButtonState.Pressed)
                    || (keyboardState.IsKeyDown(Keys.Pause) && !prevKeyboardState.IsKeyDown(Keys.Pause))
                    || (keyboardState.IsKeyDown(Keys.P) && !prevKeyboardState.IsKeyDown(Keys.P)))
                {
                    bPause = !bPause;
                }
            }
        }

        protected override void UpdateGame(GameTime gameTime)
        {
            if (state == GameState.Startup)
            {
                ChangeState(GameState.Menu);
            }

            if (state == GameState.Intro)
            {
                UpdateIntro(gameTime);
            }
            else if (state == GameState.Game)
            {
                UpdateGameState(gameTime);
            }
            else if (state == GameState.Death)
            {
                UpdateDeath(gameTime);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }

        public void ChangeState(GameState newState)
        {
            GameState oldState = state;
            state = newState;
            GameTime = new TimeSpan(0);

            if (oldState == GameState.Menu)
            {
                StarDriveLogo.RemoveFromGame();
                MainMenu.RemoveFromGame();
            }
            else if (oldState == GameState.Intro)
            {
                IntroChatBox.RemoveFromGame();
                IntroShip.RemoveFromGame();
            }
            else if (oldState == GameState.Game)
            {
            }
            else if (oldState == GameState.Death)
            {
                IntroChatBox.RemoveFromGame();
            }

            if (newState != GameState.Death)
            {
                foreach (GameObject gameObject in Layers[1].GameObjects)
                {
                    gameObject.RemoveFromGame();
                }
            }

            if (newState == GameState.Menu)
            {
                StarDriveLogo.AddToGame(this, LayerId.Foreground);
                MainMenu.position.X = StarDriveLogo.position.X - StarDriveLogo.size.X / 2 + 16;
                MainMenu.AddToGame(this, LayerId.Foreground);
            }
            else if (newState == GameState.Intro)
            {
                StartIntro();
            }
            else if (newState == GameState.Game)
            {
                StartGame();
            }
            else if (newState == GameState.Death)
            {
                StartDeath();
            }
        }

        public void StartIntro()
        {
            IntroCursor = 0;
            IntroTimer = Intro[0].time;

            IntroShip.AddToGame(this, LayerId.World);
        }

        public void UpdateIntro(GameTime gameTime)
        {
            cameraPosition = IntroShip.position;

            if (keyboardState.IsKeyDown(Keys.Escape)
                || gamePadState.IsButtonDown(Buttons.Start))
            {
                ChangeState(GameState.Game);
            }

            if (IntroCursor < Intro.Length)
            {
                if (!IntroChatBox.IsActive())
                {
                    IntroTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                    if (IntroTimer <= 0)
                    {
                        IntroChatBox.text = Intro[IntroCursor].speaker + "\n" + Intro[IntroCursor].text;
                        IntroChatBox.start = (Intro[IntroCursor].speaker + "\n").Length;
                        IntroChatBox.AddToGame(this, LayerId.Foreground);

                        IntroTimer = IntroLeadOut;
                    }
                }
                else
                {
                    if (IntroChatBox.HasFinished())
                    {
                        IntroTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                        if (IntroTimer <= 0)
                        {
                            IntroChatBox.RemoveFromGame();

                            IntroCursor++;
                            if (IntroCursor < Intro.Length)
                            {
                                IntroTimer = Intro[IntroCursor].time;
                            }
                            else
                            {
                                IntroTimer = 1.0f;
                            }
                        }
                    }
                }
            }
            else
            {
                if (IntroShip.health > 0)
                {
                    IntroTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
                    if (IntroTimer <= 0)
                    {
                        IntroShip.health = 0;
                        IntroShip.Destroy();

                        IntroTimer = 3.0f;
                    }
                }
                else
                {
                    if (IntroShip.explosion.HasEnded())
                    {
                        ChangeState(GameState.Game);
                    }
                }
            }
        }

        public void StartGame()
        {
            if (Player == null)
            {
                Player = PlayerBase.ConstructInstance<PlayerShip>();
            }
            Player.position = new Vector2(0, 0);
            Player.rotation = 0;
            Player.velocity = Vector2.Zero;
            Player.rotVelocity = 0;
            Player.AddToGame(this, LayerId.World);

            EnemySpawnTimer = EnemySpawnDelay;
            EnemyWave = 0;
        }

        public void UpdateGameState(GameTime gameTime)
        {
#if DEBUG
            if (gamePadState.IsButtonDown(Buttons.Y))
            {
                Enemy Enemy = BeamEnemyBase.ConstructInstance<Enemy>();
                Enemy.position = new Vector2(MathHelper.Lerp(-300, 300, (float)random.NextDouble()),
                                             MathHelper.Lerp(-300, 300, (float)random.NextDouble()));
                Enemy.AddToGame(this, LayerId.World);
            }
#endif

            if (Player != null && Player.IsActive())
            {
                cameraPosition = Player.position;
            }

            if (Player != null && !Player.IsActive())
            {
                ChangeState(GameState.Death);
            }

            EnemySpawnTimer -= (float)gameTime.ElapsedGameTime.TotalSeconds;
            if (EnemySpawnTimer <= 0)
            {
                EnemySpawnTimer = EnemySpawnDelay;

                SpawnEnemy();
            }
        }

        private void SpawnEnemy()
        {
            if ((float)GameTime.TotalSeconds > EnemyTimes[EnemyWave])
            {
                EnemyWave++;
            }

            Enemy newEnemy = Enemies[EnemyWave].ConstructInstance<Enemy>();
            switch(random.Next(4))
            {
                case 0:
                    newEnemy.position = cameraPosition + new Vector2(MathHelper.Lerp(-(GraphicsDevice.Viewport.Width / 2 + 16), (GraphicsDevice.Viewport.Width / 2 + 16), (float)random.NextDouble()), -GraphicsDevice.Viewport.Height / 2 - 16);
                    newEnemy.rotation = MathHelper.Pi;
                    break;
                case 1:
                    newEnemy.position = cameraPosition + new Vector2(MathHelper.Lerp(-(GraphicsDevice.Viewport.Width / 2 + 16), (GraphicsDevice.Viewport.Width / 2 + 16), (float)random.NextDouble()), GraphicsDevice.Viewport.Height / 2 - 16);
                    newEnemy.rotation = 0;
                    break;
                case 2:
                    newEnemy.position = cameraPosition + new Vector2(-GraphicsDevice.Viewport.Width / 2 - 16, MathHelper.Lerp(-(GraphicsDevice.Viewport.Height / 2 + 16), (GraphicsDevice.Viewport.Height / 2 + 16), (float)random.NextDouble()));
                    newEnemy.rotation = MathHelper.PiOver2;
                    break;
                case 3:
                    newEnemy.position = cameraPosition + new Vector2(GraphicsDevice.Viewport.Width / 2 - 16, MathHelper.Lerp(-(GraphicsDevice.Viewport.Height / 2 + 16), (GraphicsDevice.Viewport.Height / 2 + 16), (float)random.NextDouble()));
                    newEnemy.rotation = -MathHelper.PiOver2;
                    break;
            }
            newEnemy.AddToGame(this, LayerId.World);
        }

        public void StartDeath()
        {
            IntroChatBox.text = "GAME OVER.\n...\n...\nYou never did find your way home...";
            IntroChatBox.start = 0;
            IntroChatBox.AddToGame(this, LayerId.Foreground);
        }

        public void UpdateDeath(GameTime gameTime)
        {
            if (IntroChatBox.HasFinished())
            {
                if (keyboardState.IsKeyDown(Keys.Enter) && !prevKeyboardState.IsKeyDown(Keys.Enter)
                    || keyboardState.IsKeyDown(Keys.Space) && !prevKeyboardState.IsKeyDown(Keys.Space)
                    || keyboardState.IsKeyDown(Keys.Escape) && !prevKeyboardState.IsKeyDown(Keys.Escape)
                    || gamePadState.IsButtonDown(Buttons.A) && !prevGamePadState.IsButtonDown(Buttons.A)
                    || gamePadState.IsButtonDown(Buttons.Start) && !prevGamePadState.IsButtonDown(Buttons.Start))
                {
                    ChangeState(GameState.Menu);
                }
            }
        }

        public void HandleMenuItem(string menuItem)
        {
            switch (menuItem)
            {
                case @"New Game":
                    ChangeState(GameState.Intro);
                break;
                case @"Exit Game":
                    Exit();
                break;
            }
        }
    }
}
